/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.distribution.service;

import org.springblade.modules.api.vo.ExportAllAppointGoodsVO;
import org.springblade.modules.api.vo.ExportAppointGoodsVO;
import org.springblade.modules.api.vo.ExportTakeAppointGoodsVO;
import org.springblade.modules.distribution.dto.AppointDTO;
import org.springblade.modules.distribution.entity.Appoint;
import org.springblade.modules.distribution.vo.AppointStoreVO;
import org.springblade.modules.distribution.vo.AppointVO;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.core.metadata.IPage;

import java.util.List;

/**
 * 履约单表列表 服务类
 *
 * @author BladeX
 * @since 2023-06-06
 */
public interface IAppointService extends IService<Appoint> {

	/**
	 * 自定义分页
	 *
	 * @param page
	 * @param appoint
	 * @return
	 */
	IPage<AppointVO> selectAppointPage(IPage<AppointVO> page, AppointVO appoint);

	List<AppointDTO> countAppointList();

	void commitAppoint(Long appointId);

	List<ExportAppointGoodsVO> exportStoreGoods(Long grouponId, Long storeId);

	List<ExportAllAppointGoodsVO> exportAllGoods(Long grouponId);

	List<ExportTakeAppointGoodsVO> exportTakeGoods(Long grouponId, Long takeId);

	IPage<AppointStoreVO> getAppointStoreVOs(IPage<AppointStoreVO> page, Integer state, Long storeId);

	IPage<AppointStoreVO> getAppointAllVOs(IPage<AppointStoreVO> page, Integer state);

	IPage<AppointStoreVO> getAppointTakeVOs(IPage<AppointStoreVO> page, Integer state, Long takeId);

	IPage<AppointVO> appointListBy(IPage<AppointVO> page, Appoint appoint);

}
